var fs = require("fs");
var path = require("path");
/**
 * Map between uniqueId in generated file to relevant uniqueId in source file.
 * This object will be added to 'window' object due to instrumentation, in order to resolve uniquId's in browser agent
 * @constructor
 */
function SlMapper(outputPath, logger) {
    if (!validateParams(outputPath, logger)){
        return null;
    }
    this.mapping = {};
    this.filename = "sl-mapping.js";
    this.outputPath = outputPath;
    this.logger = logger;

    this.mapMethod = function(method) {
        var key = this.createUniqueIdKey(method, false);
        if(key) {
            this.mapping[key] = method.uniqueId;
        }
    }
    this.mapBranch = function(branch) {
        var key = this.createUniqueIdKey(branch, true);
        if(key) {
            this.mapping[key] = branch.uniqueId;
        }
    }


    this.getFilePath = function() {
        return path.resolve(this.outputPath, this.filename);
    }

    this.writeToFile = function () {
        var fileContent = this.createFileContent();
        var filePath = this.getFilePath();
        try {
            fs.writeFileSync(filePath, fileContent);
        }catch (e) {
            this.logger.error("Failed to write sl-mapping file Error: ", e)
        }
    }

    this.createUniqueIdKey = function(element, isBranch){
        if (element.generatedData.absoluteFilename == element.srcData.absoluteFilename
            && element.generatedData.position[0] == element.srcData.position[0] &&
            element.generatedData.position[1] == element.srcData.position[1]) {
            return null;
        }
        var delimiter = isBranch ? "|" : "@";
        // Istanbul report branches with parent location, to align the key for branches based on parent position.
        var position = isBranch ? element.generatedData.parentPosition : element.generatedData.position;
        var key = [element.generatedData.absoluteFilename, position.join(",")].join(delimiter);
        if(isBranch){
            key += "|" + element.index;
        }
        return key;
    }


    this.createFileContent = function () {
        var content ="var slMapping =" + JSON.stringify(this.mapping, null, "\t") + "\n" +
            "        if(window && (!window.$Sealights || !window.$Sealights.slMapping)){\n" +
            "            window.$Sealights = window.$Sealights  || {};\n" +
            "            window.$Sealights.slMapping = slMapping;\n" +
            "        }";
        return content;
    }

    function validateParams() {
        for(var i = 0; i< arguments.length; i++){
            if(!arguments[i]){
                return false;
            }
            return true;
        }
    }
}
module.exports.SlMapper = SlMapper;







